# gulp-sass-dynamic-importer
Gulp plugin for smart sass compilation. It will push files depending on processed partials down the stream. Supports endless stream (watch) and dynamic changes of dependency tree.

## Installation

```
npm install --save gulp-sass-dynamic-importer
```

## Usage

```coffeescript
gulp = require 'gulp'
$ = require('gulp-load-plugins')()

CSS_DIR = 'dest/styles'
SASS_DIR = 'src/styles'
SASS_CACHE = '.sass-cache'
SASS_FILES = [
    'src/styles/base/**/*.scss'
    '!src/styles/base/_bootstrap/**/*.scss'
    'src/styles/components/**/*.scss'
    'src/styles/pages/**/*.scss'
]

gulp.task 'build:sass', (done) ->
    cache = new $.fileCache(SASS_CACHE)

    gulp.src SASS_FILES, base: SASS_DIR
        .pipe cache.filter()
        .pipe cache.cache()
        .pipe $.sassDynamicImporter(dir: SASS_DIR)
        .pipe $.sass()
        .on 'error', $.sass.logError
        .pipe gulp.dest CSS_DIR

gulp.task 'watch:sass', ->
    $.watch SASS_FILES
        .pipe $.sassDynamicImporter(dir: SASS_DIR, endless: true)
        .pipe $.sass()
        .pipe gulp.dest CSS_DIR
```

### Options

|name|type|description|
|----|----|-----------|
|dir|string or string[]|Locations to consider while resolving @import paths|
|endless|boolean|By default we prevent pushing the same file twice. Set value to _true_ to support endless stream|
|logLevel|string|Allows setting more verbose logging|
