// Generated by CoffeeScript 1.10.0
var SassDependencyGraph, fs, glob, path,
  indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

fs = require('fs');

glob = require('glob');

path = require('path');

SassDependencyGraph = (function() {
  function SassDependencyGraph(log, directories) {
    this.log = log;
    this.directories = directories;
  }

  SassDependencyGraph.prototype.initialLoad = function() {
    return this.directories.forEach((function(_this) {
      return function(path) {
        return glob.sync(path + "/**/*.scss", {}).forEach(function(file) {
          if (!_this.add(file, function() {
            return fs.readFileSync(file, 'utf8');
          })) {
            return _this.log.warn("failed to add " + file + " to graph");
          }
        });
      };
    })(this));
  };

  SassDependencyGraph.prototype.add = function(filepath, getContent, parent) {
    var cwd, entry, i, importedFileContents, importedFilePath, imports, resolved;
    entry = this[filepath] = this[filepath] || {
      path: filepath,
      imports: [],
      importedBy: [],
      modified: fs.statSync(filepath).mtime
    };
    imports = this.parseImports(getContent());
    cwd = path.dirname(filepath);
    for (i in imports) {
      resolved = this.resolvePath(imports[i], this.directories.concat([cwd]));
      if (!resolved) {
        return false;
      }
      if (indexOf.call(entry.imports, resolved) < 0) {
        entry.imports.push(resolved);
        importedFilePath = path.extname(resolved) ? resolved : resolved + ".scss";
        importedFileContents = function() {
          return fs.readFileSync(importedFilePath, 'utf8');
        };
        this.add(resolved, importedFileContents, filepath);
      }
    }
    if (parent) {
      entry.importedBy.push(parent);
    }
    return true;
  };

  SassDependencyGraph.prototype.parseImports = function(content) {
    var filePath, importArgument, match, re, results, validImports;
    this.log.info('extract imports from file contents');
    re = /\@import (["'])(.+?)\1;/g;
    results = [];
    content = new String(content).replace(/\/\*.+?\*\/|\/\/.*(?=[\n\r])/g, '');
    while ((match = re.exec(content))) {
      importArgument = match[2];
      filePath = importArgument.replace(/^\.\//, '');
      results.push(filePath);
    }
    validImports = results.filter(function(x) {
      return x !== 'compass';
    }).filter(function(x) {
      return !/compass\/.*/.test(x);
    });
    this.log.info("found " + validImports.length + " valid imports");
    return validImports;
  };

  SassDependencyGraph.prototype.resolvePath = function(path, loadPaths) {
    var dotdotPattent, ext, j, len, p, partialPath, scssPath;
    for (j = 0, len = loadPaths.length; j < len; j++) {
      p = loadPaths[j];
      ext = /\.(scss|sass)$/i.test(path) ? "" : ".scss";
      scssPath = p + "/" + path + ext;
      dotdotPattent = /[^\.\/]+\/\.\.\//g;
      while (dotdotPattent.test(scssPath)) {
        scssPath = scssPath.replace(dotdotPattent, '');
      }
      if (fs.existsSync(scssPath)) {
        return scssPath;
      }
      partialPath = scssPath.replace(/\/([^\/]*)$/, '/_$1');
      if (fs.existsSync(partialPath)) {
        return partialPath;
      }
    }
    this.log.warn("failed to resolve %s from ", path, loadPaths);
    return false;
  };

  return SassDependencyGraph;

})();

module.exports = SassDependencyGraph;
