// Generated by CoffeeScript 1.10.0
var File, Importer, Options, PluginError, SassDependencyGraph, attachIcons, fs, log, normalize, obj,
  indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

fs = require('fs');

File = require('vinyl');

log = require('loglevel');

obj = require('through2').obj;

normalize = require('path').normalize;

PluginError = require('gulp-util').PluginError;

Options = require('./options');

SassDependencyGraph = require('./graph');

attachIcons = function(log) {
  var error, info, ref, success, warning;
  ref = require('log-symbols'), success = ref.success, info = ref.info, warning = ref.warning, error = ref.error;
  return {
    info: function(msg) {
      return log.info(info + " " + msg);
    },
    warn: function(msg) {
      return log.warn(warning + " " + msg);
    },
    error: function(msg) {
      return log.error(error + " " + msg);
    },
    debug: function(msg) {
      return log.debug(success + " " + msg);
    }
  };
};

Importer = (function() {
  function Importer(options1) {
    this.options = options1;
    this.alreadyPushed = [];
  }

  Importer.prototype.wasPushed = function(path) {
    if (this.options.endless) {
      return false;
    } else {
      return indexOf.call(this.alreadyPushed, path) >= 0;
    }
  };

  Importer.prototype.emitError = function(err) {
    return new PluginError('gulp-sass-dynamic-importer', err);
  };

  return Importer;

})();

module.exports = function(args) {
  var graph, importer, options;
  options = new Options(args);
  importer = new Importer(options);
  log.setLevel(options.logLevel);
  log = attachIcons(log);
  graph = new SassDependencyGraph(log, options.dir);
  log.info('graph initial load');
  graph.initialLoad();
  return obj(function(file, enc, cb) {
    var next, visitAncestors;
    next = function() {
      log.debug("exiting main function");
      return cb();
    };
    log.debug("entered main function with file [" + file.relative + "]");
    if (file.isNull()) {
      this.push(file);
      log.warn('skip null file');
      return next();
    }
    if (file.isStream()) {
      log.error('got stream instead of file');
      this.emit('error', importer.emitError('Streaming not supported'));
      return next();
    }
    visitAncestors = function(filepath, callback, visited) {
      var edges, i, ref, results;
      if (visited == null) {
        visited = [];
      }
      edges = graph[filepath].importedBy;
      results = [];
      for (i in edges) {
        if (ref = edges[i], indexOf.call(visited, ref) < 0) {
          visited.push(edges[i]);
          callback(graph[edges[i]]);
          results.push(visitAncestors(edges[i], callback, visited));
        } else {
          results.push(void 0);
        }
      }
      return results;
    };
    return fs.stat(file.path, (function(_this) {
      return function(err, stats) {
        var relativePath;
        if (err) {
          if (err.code === 'ENOENT') {
            log.info('warn about missing dependencies');
            log.warn("pass through file [" + file.path + "] that does not exist");
          } else {
            log.err("cannot access existing file [" + file.path + "]");
            _this.emit('error', importer.emitError(err));
          }
          _this.push(file);
          return next();
        }
        relativePath = file.path.substr(file.cwd.length + 1).replace(/\\/g, '/');
        if (!graph[relativePath]) {
          log.info('update graph of dependencies');
          graph.add(relativePath, function() {
            return file.contents.toString('utf8');
          });
        }
        if (relativePath.split('/').pop()[0] !== '_') {
          if (!importer.wasPushed(relativePath)) {
            log.info("processing [" + relativePath + "]");
            _this.push(file);
          }
        }
        log.info('find file dependencies and push them too');
        visitAncestors(relativePath, function(node) {
          var ancestorFile, ancestorPath;
          ancestorPath = normalize(file.cwd + "/" + node.path);
          ancestorFile = new File({
            contents: new Buffer(fs.readFileSync(ancestorPath)),
            cwd: file.cwd,
            base: file.base,
            path: node.path
          });
          if (!importer.wasPushed(node.path)) {
            log.info("processing [" + node.path + "]");
            return _this.push(ancestorFile);
          }
        });
        return next();
      };
    })(this));
  });
};
